"""
PyXLL Examples: Real time data

As well as returning static values from functions, PyXLL functions
can return special 'RTD' instances that can notify Excel of
updates to their value.

This could be used for any real time data feed, such as live
prices or the status of a service.
"""
from pyxll import RTD, xl_func, xl_app
import threading
import logging
import json
import websocket

ws = websocket.create_connection("wss://ws-feed.gdax.com")
ws.send(json.dumps({
        "type": "subscribe",
        "channels": [
            {
                "name": "ticker",
                "product_ids": ["BTC-USD"]
            }
        ]
    }))
#Check contents of result to see if subscription has succeeded
result =  ws.recv()

_log = logging.getLogger(__name__)

class GDAXRTD(RTD):

    def __init__(self):
        initial_value = ""
        super(GDAXRTD, self).__init__(value=initial_value)
        self.__running = True
        self.__thread = threading.Thread(target=self.__thread_func)
        self.__thread.start()

    def connect(self):
        # Called when Excel connects to this RTD instance, which occurs
        # shortly after an Excel function has returned an RTD object.
        _log.info("GDAXRTD Connected")

    def disconnect(self):
        # Called when Excel no longer needs the RTD instance. This is
        # usually because there are no longer any cells that need it
        # or because Excel is shutting down.
        self.__running = False
        _log.info("GDAXRTD Disconnected")

    def __thread_func(self):
        while self.__running:
            # Setting 'value' on an RTD instance triggers an update in Excel
            try:
                result =  ws.recv()
                print(result)
            except Exception as e:
                result = e
            self.value = result

@xl_func(": rtd")
def gdax_rtd():   
    return GDAXRTD()